@make(ais)
@aiInfo(topic=<Allow 256 values for type CHARACTER>,
number=[AI-00420/03],
classcode=[ST],
statuscode=[RE],
mainref=[02.05@\(01)],
date=[88-11-08])
@aiHeading()
@begin(history)
@begin(toplines)
@mainref()
@standard[02.01@\(01)]
@standard[03.05.02@\(01)]
@standard[04.02@\(05)]
@standard[C        (13)]
!class study 85-06-18
!status received 85-06-18
@end(toplines)
@topic{!topic Allow 256 values for type CHARACTER}
@end(history)

@subheading[!summary 86-05-05]


@subheading[!question 86-05-05]

In order to allow non-English characters in string literals, the predefined
type CHARACTER should be extended to an eight-bit character set.

@subheading[!recommendation 86-05-05]


@subheading[!discussion 86-05-05]


@begin(appendix)
@subheading[!appendix @cbon()88-10-20]@cboff()
@begin(sectionGroup)
@mybar()

!section 02.01    (01) Japanese comments on DP8652 85-05-10          83-00557
!version 1983
!topic Extend character code to allow non-English characters
@end(sectionGroup)

The Standard says, "Each graphic character corresponds to a unique code of the
ISO seven-bit coded character set (ISO standard 646)."  This specification
reduces applicability for processing non-English characters (for example,
Japanese Kana, Korean characters, Arabic characters).  We propose a extended
specification of character set as follows:

The only characters allowed in the text of a program are the graphic characters
and format effectors.  Each graphic character corresponds to a unique code of
the ISO eight-bit coded character set (ISO standard 4873), and represented
(visually) by a graphic symbol.

The characters of columns 00 to 07 of the ISO eight-bit coded character table
are categorized by the same rule as the present Standard.  The defined graphic
characters included by columns 10 to 15 are categorized in "(f) other special
characters".  The other characters are not allowed in the text of a program.

The predefined type CHARACTER is a character type whose values are 256
characters of the ISO eight-bit coded character set.

The reason for this recommendation is to enforce the applicability for
processing non-English characters.

@begin(sectionGroup)
@mybar()

!section 02.01    (01) R. Leavitt  86-11-24                          83-00868
!version 1983
!topic French characters in strings
@end(sectionGroup)

Canada requires its command and control programs to interact with the
operator in both English and French.  This appears to be difficult in
Ada.  Ada strings cannot contain all the characters required for French
text.  How can portable programs be written that prompt the operator in
any language other than English?

@begin(sectionGroup)
@mybar()

!section 02.01    (00) Japanese Member Body/ISO 88-02-01             83-01014
!version 1983
!topic Requirements for character handling in programming languages
@end(sectionGroup)
!reference ISO/TC97/SC22/N460

BASIC REQUIREMENTS

1.  Every programming language standard shall have optional facilities both to
    handle multi-octet characters as data and to allow multi-octet characters
    in comments.

2.  In addition to the above it is highly desirable that programming language
    standards have optional facilities to allow multi-octet characters in
    identifiers.

JAPANESE CHARACTER SET

Japanese character set is composed of Kanji, Hiragana and Katakana
characters.  JIS (Japanese Industrial Standard) X0208--Code of the Japanese
Graphic Character Set for Information Interchange--has totally 6877
characters, including 6353 Kanji, 189 Hiragana and Katakana, 147 special
symbols (including most of the ASCII special symbols), and 62 alphanumeric
characters.  Each character in JIS X0208 code set is represented in two-byte
code.

Based on this code set for information interchange, there are several
Japanese code sets for internal processing.

DATA TYPES

We need at least two character types, one (Type I) for the ISO 646 (or its
akin like ASCII) character set, and the other (Type II) for a national
character set.  Both types shall be usable in one program unit.  In general,
a character data type shall be characterized by the length of memory space
occupied by a character, a single-byte for a Type I character and a
multi-byte (or a single-byte) for a Type II character.  In our case, the
Type II character set is the Japanese character set and a Type II character
occupies a two-byte memory space.

It may be possible to provide only one character type, Type II, in some
programming languages if the Type II character set includes the Type I
character set.  In such a case, it may also be possible to use single-byte
codes for the Type I characters as optimized representations.

In some programming languages where the major concern is flexibility rather
than efficiency, it may be desirable to characterize the type not by memory
spaces but by character sets only.

CHARACTERS IN DATA

The following facilities are required:

  As internal data

    1.  The character type and the string (or array of character) type for both
        Type I and Type II are provided.

    2.  Type I data (single-byte codes) and Type II data (multi-byte codes) are
        assumed not to be mixed within a character type variable.

    3.  Data length is counted by the number of characters not by the number of
        bytes.

    4.  It is desirable that the same set of standard functions for character
        data are provided, using the same function name as far as possible,
        for both Type I and Type II.

    5.  Conversion functions for common characters between Type I data and
        Type II data are provided, if both types have considerable number of
        common characters, as in our case.

    As data in program text

    Two forms of character literals for Type I and Type II are provided.

    Examples: "East West South North" -- Type I
               N" (japanese symbols)" -- Type II

  As external data

    1.  Mixture of Type I and II characters is allowed in external files.  In
        general, it is desirable that only the input/output procedures handle
        this mixture.  Therefore, it is desirable, for example, that the input
        procedures assign their values separately to appropriate (Type I or
        II) internal variables according to the format specifications in the
        input statements.  However, in some programming languages, especially
        in languages for system programs, it may be required that some
        variables have mixed type strings.  The type of such a variable may be
        neither Type I nor Type II, but type or integer.

    2.  Usually, the Kanji characters are twice as wide as alphanumeric
        characters in printed form and we have two widths (normal-width and
        double-width) of alphanumeric characters in Japanese word processors.
        However, in general, the character width shall have nothing to do with
        the type of characters in output formatting.  The character width
        shall be specified by some kind of format specification or by default,
        for example, normal-width for the Type I and double-width for the Type
        II except Katakana.

Comments

  The type II character shall be allowed in comments.

Identifiers

  It is desirable that the Type II characters are allowed in identifiers.

Keywords

  Keywords shall be composed only of the Type I characters.

@cbon()@begin(sectionGroup)
@mybar()

!section 02.01    (00) David B. Kinder, BiiN 88-10-20                83-01026
!version 1983
!topic 8-bit character type
@end(sectionGroup)

AI-420 (submitted over 3 years ago) brought up the issue of an an 8-bit
character type.  This is now becoming a hot issue with us and our
Non-US customers who are insisting on support for their (non-English)
characters in STRING variables and in TEXT_IO.  I've seen packages
that define a new extended character type, however this character type
can't be used with TEXT_IO, so a whole new (and non-standard) I/O package
must be used.

Have there been any further thoughts by the LMC/ARG on supporting 8-bit
characters?  Can we get some ruling on AI-420? Is ISO 4873 being considered?

-- David Kinder, BiiN@cboff()

@end(appendix)
